using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace DriveFast
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        private Texture2D mRoad;

        private float mVelocityY;
        private int[] mRoadY = new int[3];

        private Texture2D mCar;
        private Rectangle mCarPosition;

        private KeyboardState mPreviousKeyboardState;
        private int mMoveCarX;

        private GamePadState mPreviousGamePadState;

        private Texture2D mHazard;
        private Rectangle mHazardPosition;

        private Random mRandom = new Random();

        private int mHazardsPassed;
        private SpriteFont mFont;

        private enum State
        {
            Running,
            Crash
        }

        private State mCurrentState;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
            StartGame();
        }

        protected void StartGame()
        {
            mRoadY[0] = 0;
            mRoadY[1] = mRoadY[0] + -mRoad.Height + 2;
            mRoadY[2] = mRoadY[1] + -mRoad.Height + 2;

            mVelocityY = 0.3F;

            mCarPosition = new Rectangle(280, 440, (int)(mCar.Width * 0.2f), (int)(mCar.Height * 0.2f));
            mMoveCarX = 160;

            mHazardPosition = new Rectangle(275, -mHazard.Height, (int)(mHazard.Width * 0.4f), (int)(mHazard.Height * 0.4f));

            mHazardsPassed = 0;

            mCurrentState = State.Running;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            mRoad = Content.Load<Texture2D>("Road");
            mCar = Content.Load<Texture2D>("Car");
            mHazard = Content.Load<Texture2D>("Hazard");

            mFont = Content.Load<SpriteFont>("myFont");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState aCurrentKeyboardState = Keyboard.GetState();
            GamePadState aCurrentGamePadState = GamePad.GetState(PlayerIndex.One);

            if (aCurrentKeyboardState.IsKeyDown(Keys.Escape) == true || aCurrentGamePadState.Buttons.Back == ButtonState.Pressed)
            {
                this.Exit();
            }

            switch (mCurrentState)
            {
                case State.Running:
                    {
                        if ((aCurrentKeyboardState.IsKeyDown(Keys.Space) == true && mPreviousKeyboardState.IsKeyDown(Keys.Space) == false)
                            || (aCurrentGamePadState.Buttons.X == ButtonState.Pressed && mPreviousGamePadState.Buttons.X == ButtonState.Released))
                        {
                            mCarPosition.X += mMoveCarX;
                            mMoveCarX *= -1;
                        }

                        ScrollRoad(gameTime);

                        if (mHazardPosition.Intersects(mCarPosition) == false)
                        {
                            UpdateHazard(gameTime);
                        }
                        else
                        {
                            mCurrentState = State.Crash;
                        }

                        break;
                    }
                case State.Crash:
                    {
                        if (aCurrentKeyboardState.IsKeyDown(Keys.Enter) == true || aCurrentGamePadState.Buttons.Start == ButtonState.Pressed)
                        {
                            StartGame();
                        }
                        break;
                    }
            }

            mPreviousKeyboardState = aCurrentKeyboardState;
            mPreviousGamePadState = aCurrentGamePadState;

            base.Update(gameTime);
        }

        private void ScrollRoad(GameTime theTime)
        {
            //Loop the road
            for (int aIndex = 0; aIndex < mRoadY.Length; aIndex++)
            {
                if (mRoadY[aIndex] >= this.Window.ClientBounds.Height)
                {
                    int aLastRoadIndex = aIndex;
                    for (int aCounter = 0; aCounter < mRoadY.Length; aCounter++)
                    {
                        if (mRoadY[aCounter] < mRoadY[aLastRoadIndex])
                        {
                            aLastRoadIndex = aCounter;
                        }
                    }

                    mRoadY[aIndex] = mRoadY[aLastRoadIndex] - mRoad.Height + 2;
                }
            }

            //Move the road
            for (int aIndex = 0; aIndex < mRoadY.Length; aIndex++)
            {
                mRoadY[aIndex] += (int)(mVelocityY * theTime.ElapsedGameTime.TotalMilliseconds);
            }
        }

        private void UpdateHazard(GameTime theTime)
        {
            mHazardPosition.Y += (int)(mVelocityY * theTime.ElapsedGameTime.TotalMilliseconds);

            if (mHazardPosition.Y > this.Window.ClientBounds.Height)
            {
                mHazardPosition.X = 275;
                if (mRandom.Next(1, 3) == 2)
                {
                    mHazardPosition.X = 440;
                }

                mHazardPosition.Y = -mHazard.Height;

                mHazardsPassed += 1;
                mVelocityY += 0.1F;
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.SaddleBrown);

            // TODO: Add your drawing code here
            spriteBatch.Begin();

            for (int aIndex = 0; aIndex < mRoadY.Length; aIndex++)
            {
                spriteBatch.Draw(mRoad, new Vector2(0, mRoadY[aIndex]), Color.White);
            }

            spriteBatch.Draw(mCar, mCarPosition, Color.White);
            spriteBatch.Draw(mHazard, mHazardPosition, Color.White);

            spriteBatch.DrawString(mFont, "Hazards: " + mHazardsPassed.ToString(), new Vector2(5, 25), Color.White,
                0, new Vector2(0, 0), 1.0f, SpriteEffects.None, 0);

            if (mCurrentState == State.Crash)
            {
                spriteBatch.DrawString(mFont, "Crash!", new Vector2(5, 200), Color.White, 0, new Vector2(0, 0), 1.0f,
                    SpriteEffects.None, 0);
                spriteBatch.DrawString(mFont, "'Enter' to play again.", new Vector2(5, 230), Color.White, 0,
                    new Vector2(0, 0), 1.0f, SpriteEffects.None, 0);
                spriteBatch.DrawString(mFont, "'Escape' to exit.", new Vector2(5, 260), Color.White, 0,
                    new Vector2(0, 0), 1.0f, SpriteEffects.None, 0);
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
